package com.me.brickbuster.entity;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.glutils.ShapeRenderer.ShapeType;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Vector2;
import com.me.brickbuster.BrickBuster;
import com.me.brickbuster.Utils;
import net.dermetfan.utils.Pair;

import java.util.Iterator;

public class Ball extends Entity {

	public static final int RADIUS = 12;
	public static final Color BALL_COLOR = Color.CHARTREUSE;

	public Vector2 direction;
	public float speed;

	public Ball(BrickBuster brickBuster) {
		super(brickBuster,BrickBuster.WIDTH/2, Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + RADIUS);
	}

	@Override
	public void render() {
		getShapeRenderer().begin(ShapeType.Filled);
		getShapeRenderer().setColor(BALL_COLOR);
		getShapeRenderer().circle(getPos().x, getPos().y, RADIUS);
		getShapeRenderer().end();
	}

	@Override
	public void update(float dt) {
		if (!getBrickBuster().isPlaying()) {
			return;
		}

		Vector2 new_pos = getPos().cpy().add(direction.cpy().scl(speed));

		boolean brick_collision = false;
		Iterator<Brick> brickIterator = getBrickBuster().getBricks().iterator();
		while (!brick_collision && brickIterator.hasNext()) {
			Brick brick = brickIterator.next();
			Vector2[] vertices = brick.getVertices();
			for(int i = 0; i < vertices.length; i++) {
				Vector2 v1 = vertices[i];
				Vector2 v2 = vertices[i+1 < vertices.length? i+1 : 0];
				Vector2 segment = v2.cpy().sub(v1);
				Vector2 nearest = Utils.nearestPoint(v1.cpy(), segment, new_pos.cpy());

				if (nearest.dst(new_pos.x, new_pos.y) <= RADIUS) {
					brickIterator.remove();
					if (getBrickBuster().getBricks().size() <= 39) {
						speed = 8;
					}
					Utils.reflect(direction, segment.nor());
					brick_collision = true;
					break;
				}
			}
		}

		if (new_pos.x + RADIUS > BrickBuster.WIDTH || new_pos.x - RADIUS < 0) {
			Utils.reflect(direction, BrickBuster.VERTICAL_EDGE);
		} else if (new_pos.y + RADIUS > BrickBuster.HEIGHT) {
			Utils.reflect(direction, BrickBuster.HORIZONTAL_EDGE);
		} else if (new_pos.y - RADIUS < 0) {
			reset();
			return;
		}

		if (direction.y < 0) {
			Pair<Vector2, Vector2> paddle = getBrickBuster().getPaddle().getTopEdge();
			Vector2 lineDir = paddle.getValue().sub(paddle.getKey());
			Vector2 nearest = Utils.nearestPoint(paddle.getKey().cpy(), lineDir, new_pos.cpy());

			if (nearest.dst(new_pos.x, new_pos.y) <= RADIUS) {
				paddleCollision();
			}
		}

		getPos().add(direction.cpy().scl(speed));
	}

	public void launch() {
		float angle = MathUtils.random(MathUtils.PI/2) + MathUtils.PI/4;
		direction = new Vector2(MathUtils.cos(angle), MathUtils.sin(angle));
		speed = 5;
	}

	public void paddleCollision() {
		float paddleCenter = getBrickBuster().getPaddle().getX() + Paddle.PADDLE_WIDTH/2;
		float rel = MathUtils.clamp((getX() - paddleCenter) + 50, 5, 95);
		float newAngle = MathUtils.PI - (MathUtils.PI * (rel/100));
		direction = new Vector2(MathUtils.cos(newAngle), MathUtils.sin(newAngle));
	}

	public void reset() {
		setX(getBrickBuster().getPaddle().getX() + Paddle.PADDLE_WIDTH/2);
		setY(Paddle.PADDLE_Y + Paddle.PADDLE_HEIGHT + RADIUS);
		getBrickBuster().setPlaying(false);
	}
}
